/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2006
 *
 *    File name   : main.c
 *    Description : Define main module
 *
 *    History :
 *    1. Data        :
 *       Author      : Stanimir Bonev
 *       Description : 3, January 2006
 *
 * Jumpers:
 *  EXT/JLINK - the correct jumper position depends on the power source used
 *  DBG       - install the jumper
 *  INT0      - install the jumper
 *  INT1      - install the jumper
 *  INT2      - install the jumper
 * Buttons:
 *  INT0      - Decrement/Backlight On
 *  INT1      - Correct time/Backlight On
 *  INT2      - Increment/Backlight On
 *
 *    $Revision: 1.2.2.2 $
 **************************************************************************/
#include "includes.h"

#define DLY_100US       100

volatile Boolean rKeyUpPress, rKeyDownPress, rKeySetPress;


void HandleError(unsigned char cc);

/*************************************************************************
 * Function Name: irq_handler
 * Parameters: none
 *
 * Return: none
 *
 * Description: IRQ handler
 *
 *************************************************************************/
__irq __arm void irq_handler (void)
{
void (*interrupt_function)();
unsigned int vector;

  vector = VICVectAddr;     // Get interrupt vector.
  interrupt_function = (void(*)())vector;
  if(interrupt_function != NULL)
  {
    interrupt_function();  // Call vectored interrupt function.
  }
  else
  {
    VICVectAddr = 0;      // Clear interrupt in VIC.
  }
}

/*************************************************************************
 * Function Name: KeyUpIntHandler
 * Parameters: none
 *
 * Return: none
 *
 * Description: Key UP interrupt handler
 *
 *************************************************************************/
void KeyUpIntHandler (void)
{
  EXTINT = 4;
  rKeyUpPress = TRUE;
  VICVectAddr = 0;
}

/*************************************************************************
 * Function Name: KeySetIntHandler
 * Parameters: none
 *
 * Return: none
 *
 * Description: Key SET interrupt handler
 *
 *************************************************************************/
void KeySetIntHandler (void)
{
  EXTINT = 2;
  rKeySetPress = TRUE;
  VICVectAddr = 0;
}

/*************************************************************************
 * Function Name: KeyDownIntHandler
 * Parameters: none
 *
 * Return: none
 *
 * Description: Key DOWN interrupt handler
 *
 *************************************************************************/
void KeyDownIntHandler (void)
{
  EXTINT = 1;
  rKeyDownPress = TRUE;
  VICVectAddr = 0;
}

/*************************************************************************
 * Function Name: KeysInit
 * Parameters: none
 *
 * Return: none
 *
 * Description: Init Keys
 *
 *************************************************************************/
void KeysInit (void)
{
  // Init variables
  rKeyUpPress = rKeyDownPress = rKeySetPress = FALSE;
  // Set to EXT Interrupt
  PINSEL0_bit.P0_14 = 1;
  PINSEL0_bit.P0_15 = 1;
  PINSEL1_bit.P0_16 = 1;
  // Set ext. interrupt edge to sensitive mode
  EXTMODE = 0x7;
  // Falling Edge
  EXTPOLAR = 0;
  // Set interrupts
  // Assign to IRQ
  VICIntSelect_bit.EINT0 =\
  VICIntSelect_bit.EINT1 =\
  VICIntSelect_bit.EINT2 = 0;
  // Set slots
  VICVectAddr0 = (Int32U) KeyDownIntHandler;
  VICVectCntl0_bit.NUMBER = VIC_EINT0;
  VICVectCntl0_bit.ENABLED = TRUE;
  VICVectAddr1 = (Int32U) KeySetIntHandler;
  VICVectCntl1_bit.NUMBER = VIC_EINT1;
  VICVectCntl1_bit.ENABLED = TRUE;
  VICVectAddr2 = (Int32U) KeyUpIntHandler;
  VICVectCntl2_bit.NUMBER = VIC_EINT2;
  VICVectCntl2_bit.ENABLED = TRUE;
  // Clear pending interrupts
  EXTINT = 0x7;
  // Enable interrupts
  VICIntEnable_bit.EINT0 =\
  VICIntEnable_bit.EINT1 =\
  VICIntEnable_bit.EINT2 = 1;
}

/*************************************************************************
 * Function Name: Dly100us
 * Parameters: void *arg
 *
 * Return: none
 *
 * Description: Delay ~ (arg * 100us)
 *
 *************************************************************************/
void Dly100us (void *arg)
{
  for(volatile Int32U i = (Int32U)arg; i; i--)
  {
    for(volatile Int32U j = DLY_100US; j; j--);
  }
}

/*************************************************************************
 * Function Name: main
 * Parameters: none
 *
 * Return: none
 *
 * Description: main
 *
 *************************************************************************/
void main(void)
{
Boolean CursorOn;                 // Cursor state flag
HD44780_XY_DEF CursorX, CursorY;  // Hold cursor position
//Int8S Line1[17], Line2[17];       // Line 1,2


  // Disable PLL
  PLLCON = 0;
  // Write Feed
  PLLFEED = 0xAA;
  PLLFEED = 0x55;
  // Set periphery divider /4
  APBDIV_bit.APBDIV  = 0;
  // Set MAM fully enable
  MAMCR_bit.MODECTRL = 0;
  MAMTIM_bit.CYCLES  = 3;
  MAMCR_bit.MODECTRL = 2;
  // Memory map init flash memory is maped on 0 address
  MEMMAP_bit.MAP = 1;
  // Init VIC
  __disable_interrupt();
  // Assign all interrupt chanels to IRQ
  VICIntSelect  =  0;
  // Diasable all interrupts
  VICIntEnClear = 0xFFFFFFFF;
  // Clear all software interrutps
  VICSoftIntClear = 0xFFFFFFFF;
  // VIC registers can be accessed in User or privileged mode
  VICProtection = 0;
  // Clear interrupt
  VICVectAddr = 0;
  // Clear address of the Interrupt Service routine (ISR) for non-vectored IRQs.
  VICDefVectAddr = 0;
  // Clear address of the Interrupt Service routine (ISR) for vectored IRQs.
  VICVectAddr0  = VICVectAddr1  = VICVectAddr2  = VICVectAddr3  =\
  VICVectAddr4  = VICVectAddr5  = VICVectAddr6  = VICVectAddr7  =\
  VICVectAddr8  = VICVectAddr9  = VICVectAddr10 = VICVectAddr11 =\
  VICVectAddr12 = VICVectAddr13 = VICVectAddr14 = VICVectAddr15 = 0;
  // Disable all vectored IRQ slots
  VICVectCntl0  = VICVectCntl1  = VICVectCntl2  = VICVectCntl3  =\
  VICVectCntl4  = VICVectCntl5  = VICVectCntl6  = VICVectCntl7  =\
  VICVectCntl8  = VICVectCntl9  = VICVectCntl10 = VICVectCntl11 =\
  VICVectCntl12 = VICVectCntl13 = VICVectCntl14 = VICVectCntl15 = 0;
  // Init GPIO
  KeysInit();
  // Init RTI
  RtcInit();
  // Init LCD
  __enable_interrupt();
  HD44780_PowerUpInit();
  // Init variables
  HD44780SetBackLight(TRUE);

  // Show text on LCD
  HD44780_StrShow(1,1,"Eingebettete    ");
  HD44780_StrShow(1,2,"Systeme         ");
  
  
  //visualSTATE
  /* Define completion code variable. */
  unsigned char cc;

  /* Define action expression variable. */
  SEM_ACTION_EXPRESSION_TYPE actionExpressNo;
  
  /* Define and initialize. */
  SEM_EVENT_TYPE eventNo;
  
  /* Initialize the VS System. */
  //SEM_InitAll();
  SEM_Init();
  /* Initialize the VS Event queue. */
  SEQ_Initialize();
  
  /* Add SE_RESET to initialize system. */
  SEQ_AddEvent(SE_RESET);
  
  
  // main loop
  while(1)
  {
   
    if(SEQ_RetrieveEvent(&eventNo) != UCC_QUEUE_EMPTY)
    {     
      /* Deduct the event. */
      if ((cc = SEM_Deduct(eventNo)) != SES_OKAY) 
        HandleError(eventNo);
      
      /* Get resulting action expressions and execute them. */
      while ((cc = SEM_GetOutput(&actionExpressNo)) == SES_FOUND) 
        SEM_Action(actionExpressNo);
            
      /* Check for error. */
      if (cc != SES_OKAY) 
        HandleError(cc);
      
      /* Change the next state vector. */
      if ((cc = SEM_NextState()) != SES_OKAY) 
        HandleError(cc);    
    }    
    if(rKeyDownPress){
      rKeyUpPress  = rKeyDownPress = rKeySetPress = FALSE;
      //sprintf((char*)Line2, "  rKeyDownPress  ");
      SEQ_AddEvent(Key0Press);
    }
  
    if(rKeySetPress){
      rKeyUpPress  = rKeyDownPress = rKeySetPress = FALSE;
      //sprintf((char*)Line2, "  rKeySetPress   ");
      SEQ_AddEvent(Key1Press);
    }
      if(rKeyUpPress){
      rKeyUpPress  = rKeyDownPress = rKeySetPress = FALSE;
      //sprintf((char*)Line2, "  rKeyUpPress    ");
      SEQ_AddEvent(Key2Press);
    }

      HD44780_CursorPosSet(FALSE,FALSE,1,1);

      //HD44780_StrShow(1,2,Line2);
      if(CursorOn)
      {
        // Dispaly cursor
        HD44780_CursorPosSet(TRUE,TRUE,CursorX,CursorY);
      } // if(CursorOn)
  } // while(1)
} // main(void)
